#!/usr/bin/perl
#
# nephchart - Honestly, I didn't know about Fred Schiff's version
# of NephCal when I wrote this.  His has much prettier output and is MUCH
# more portable, but for those of you on Unix machines (see, I used the 'cal'
# command, below), this program condenses and makes available a lot more of
# the non-pertinent details like what sign of the Zodiac is connected with
# which date or which planets are influencing the magical fields.  It also
# checks to see if there should be a Nexus or a Plexus somewhere
# in the world, and on days of Grand Enthronement, it checks to see if
# Nephilim with a dominant and opposed Ka-element should make a roll
# for Khaiba -- I've gone with what it says on p. 122 of the rulebook and
# NOT what it says on p. 130, so Khaiba rolls only happen when the modifier
# is 9 or greater ... 7 or greater is just too often.
#
# usage: nephchart [year]
#
# The output should be pretty easy to interpret.  If an Enthronement is
# marked with an asterisk, it's a Grand Enthronement.  If it has a tilde,
# it's a Neutral Enthronement.  If it has a number/pound sign, it's an
# Opposed Enthronement.  The Planetary influences are listed first by day
# of the week, then by sign of the Zodiac.  The modifiers are abbreviated as
# F[ire], A[ir], E[arth], W[ater], and M[oon].  You get the idea.
#
srand(time);

$year=pop(@ARGV) || die "Please supply an A.D. year as an argument\n";
if ($year=~/[^0-9]/) {die "Bad argument\n";}
if ($year > 9999) {die "Bad argument\n";}
if ($year%4 == 0) {$leap=1;}
if ($year == 1752) {$weird=1;} #Ignore results for Sep 3-13, 1752.

sub spaces {
  $how_many=pop(@_);
  for ($n=1; $n <= $how_many; $n++) {print " ";}
}

sub d100 {return int(rand(100))+1;}
sub d10  {return int(rand(10))+1;}
sub d6   {return int(rand(6))+1;}
sub d4   {return int(rand(4))+1;}

%day_of_week = (
  '1', 'Sun:Sun',
  '2', 'Mon:Moon',
  '3', 'Tue:Mars',
  '4', 'Wed:Mercury',
  '5', 'Thu:Jupiter',
  '6', 'Fri:Venus',
  '7', 'Sat:Saturn',
);

%planet_field = (
  'Moon', 'Moon',
  'Mars', 'Fire',
  'Mercury', 'Air',
  'Jupiter', 'Earth',
  'Venus', 'Water',
  'Saturn', 'Orich',
);

%zodiac = (
  'Jan 1',  'Capricorn:Saturn',
  'Jan 20', 'Aquarius:Saturn',
  'Feb 19', 'Pisces:Jupiter',
  'Mar 21', 'Aries:Mars',
  'Apr 21', 'Taurus:Venus',
  'May 21', 'Gemini:Mercury',
  'Jun 22', 'Cancer:Moon',
  'Jul 23', 'Leo:Sun',
  'Aug 23', 'Virgo:Mercury',
  'Sep 23', 'Libra:Venus',
  'Oct 23', 'Scorpio:Mars',
  'Nov 22', 'Sagittarius:Jupiter',
  'Dec 21', 'Capricorn:Saturn',
);

# Bonuses are in the order Fire:Air:Earth:Water:Moon
%day_bonuses = (
  'Moon',    '-:-: : :+',
  'Mars',    '+: : :-:-',
  'Mercury', ' :+:-: :-',
  'Jupiter', ' :-:+:-: ',
  'Venus',   '-: :-:+: ',
  'Saturn',  '-:-:-:-:-',
  'Sun',     ' : : : : ',
);

%mag_d6 = (
  'Moon:Venus', '1',
  'Moon:Jupiter', '1',
  'Moon:Saturn', '1',
  'Mars:Mercury', '1',
  'Mars:Jupiter', '1',
  'Mars:Saturn', '1',
  'Mercury:Mars', '1',
  'Mercury:Venus', '1',
  'Mercury:Saturn', '1',
  'Jupiter:Mars', '1',
  'Jupiter:Moon', '1',
  'Jupiter:Saturn', '1',
  'Venus:Mercury', '1',
  'Venus:Moon', '1',
  'Venus:Saturn', '1',
);

open(CAL,"cal $year |") || die "Could not get calendar\n";
while(<CAL>) {
  if (!$first_day) {
    if (/\ 1\ /) {
      $col=index($_,"\ 1\ ");
      $first_day=($col/3)+1; 
    }
  }
}

print "\n                       Astrological modifiers for $year\n\n";
print " Date  Day    Sign     Day/Sign Influences   Enthr.         Modifiers\n";
print "====== === =========== ===================== ====== =========================\n";

$day=1;
$dow=$first_day;
$mon='Jan';
if ($leap) {$max_days=366;}
else {$max_days=365;}
@planets=('Moon','Mars','Mercury','Jupiter','Venus','Saturn');

for ($c=1; $c <= $max_days; $c++) {
  $leo=0;
  $date=join(' ',$mon,$day);
  ($how_long,$dlong,$zlong)=0;
  if ($zodiac{$date}) {$zomo=$zodiac{$date};}
  ($zoname,$zoplanet)=split(':',$zomo);
  ($dayname,$dayplanet)=split(':',$day_of_week{$dow});
  if ($dayplanet eq 'Sun') {
    $dtmp=splice(@planets, rand @planets, 1);
    push(@planets,$dtmp);
  }
  if ($zoplanet eq 'Sun') {
    $ztmp=splice(@planets, rand @planets, 1);
    push(@planets,$ztmp);
    $leo=1;
  }
  print $date;
  &spaces(7-length($date));
  print $dayname," ",$zoname;
  &spaces(12-length($zoname));
  print $dayplanet;
  $how_long=length($dayplanet.$zoplanet.'+');
  if ($dtmp) {print "($planet_field{$dtmp})"; $dayplanet=$dtmp;
    $dlong=length($planet_field{$dtmp}.'()'); $dtmp=0;}
  print "+",$zoplanet;
  if ($ztmp) {print "($planet_field{$ztmp})"; $zoplanet=$ztmp;
    $zlong=length($planet_field{$ztmp}.'()'); $ztmp=0;}
  $how_long=$how_long+$dlong+$zlong;
  &spaces(22-$how_long); 
  if ($dayplanet eq $zoplanet) {$enth='*'.$planet_field{$dayplanet};}
  elsif ($mag_d6{$dayplanet.':'.$zoplanet}) {
    $enth='~'.$planet_field{$dayplanet};
  }
  else {$enth='#'.$planet_field{$dayplanet};}
  print $enth;
  &spaces(7-length($enth));
  $daybonus=$day_bonuses{$dayplanet};
  $mod=&d4;
  if ($mag_d6{$dayplanet.':'.$zoplanet}) {$mod=&d6;}
  if (($dayplanet eq $zoplanet)||($leo)) {$mod=&d10;}
  @ka=('F','A','E','W','M');
  @modifiers=split(':',$daybonus);
  while (@modifiers) {
    print shift(@ka);
    $_=shift(@modifiers);
    if (/\+/) {
      if ($zoplanet ne 'Saturn') {print "+",$mod;}
      else {print "-",$mod;}
    }
    if (/\-/) {print "-",$mod;}
    if (/\ /) {
      if ($zoplanet ne 'Saturn') {print ' 0'; if ($mod == 10) {print " ";}}
      else {print "-",$mod;}
    }
    print " ";
    if ($mod < 10) {print " ";}
  }
  print "\n";

  if (($mod > 8)&&($dayplanet eq $zoplanet)&&($zoplanet ne 'Saturn')) {
    $enth=~s/\W//g;
    print "    ** Nephilim with a Dominant Ka-element opposed to ";
    print "$enth roll for Khaiba.\n";
  }

  $exus=0;
  if (($mod > 5)&&($zoplanet ne 'Saturn')&&($dayplanet ne 'Saturn')) {
    $exuscheck=&d100;
    if ($exuscheck < $mod) {
      $enth=~s/\W//g;
      print "    ** A Plexus of $enth-Ka may exist.";
      $exus=1;
    }
    if ($exuscheck == $mod) {print "    ** A Nexus may exist!"; $exus=1;}
    if ($exus) {
      print "  Its duration roll was ",&d100," on a d100.\n";
    }
  }

# An Orichalka Plexus is "very rare."  Same % chance as a Nexus, which
# is also supposed to be "very rare." 

  if (($mod > 5)&&($dayplanet eq 'Saturn')) {
    $exuscheck=&d100;
    if ($exuscheck == $mod) {
      print "    ** An Orichalka Plexus may exist!";
      print "  Its duration roll was ",&d100," on a d100.\n";
    }
  }

  $day++;
  $dow++;
  if ($dow > 7) {$dow=1;}
  if (($mon eq 'Nov')&&($day==31)) {$mon='Dec'; $day=1;}
  if (($mon eq 'Oct')&&($day==32)) {$mon='Nov'; $day=1;}
  if (($mon eq 'Sep')&&($day==31)) {$mon='Oct'; $day=1;}
  if (($mon eq 'Sep')&&($day==3)&&($weird)) {$day=14; $max_days=$max_days-11;}
  if (($mon eq 'Aug')&&($day==32)) {$mon='Sep'; $day=1;}
  if (($mon eq 'Jul')&&($day==32)) {$mon='Aug'; $day=1;}
  if (($mon eq 'Jun')&&($day==31)) {$mon='Jul'; $day=1;}
  if (($mon eq 'May')&&($day==32)) {$mon='Jun'; $day=1;}
  if (($mon eq 'Apr')&&($day==31)) {$mon='May'; $day=1;}
  if (($mon eq 'Mar')&&($day==32)) {$mon='Apr'; $day=1;}
  if (($mon eq 'Feb')&&($day==30)) {$mon='Mar'; $day=1;}
  if (($mon eq 'Feb')&&($day==29)&&(!$leap)) {$mon='Mar'; $day=1;}
  if (($mon eq 'Jan')&&($day==32)) {$mon='Feb'; $day=1;}
}
